/*
** Module   : SelectionListWidget.java
** Abstract : server side selection list widget implementation
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -------------------Description--------------------
** 001 NVS 20051026   @23495 Created initial version.
** 002 SIY 20060209   @24429 Major refactoring.
** 003 SIY 20060210   @24514 Now isSelected() methods using real selection 
**                           data. Added setSingle() convenience method.
** 004 GES 20060803   @28383 Some cleanup and refactoring.
** 005 GES 20080320   @37594 Added TYPE attribute.
** 006 GES 20110422          Added scrollbar flag getter/setter methods.
** 007 SIY 20111101          Updated imports.
** 008 CA  20130221          Refactored usage of TYPE attribute.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Server side selection list widget implementation.
 */
public class SelectionListWidget
extends ControlSetEntity
{
   /** Reference to our configuration which is pre-cast to the proper type. */
   private SelectionListConfig slc = null;
   
   /**
    * Default constructor.
    */
   public SelectionListWidget()
   {
      super(new SelectionListConfig(ComponentConfig.SELECTION_LIST, null));
      slc = (SelectionListConfig) config;
      
      setResourceType("SELECTION-LIST");
   }
   
   /**
    * Getter for the INNER-CHARS writeable attribute.
    *
    * @return current value of INNER-CHARS attribute
    */
   public integer getInnerChars()
   {
      return new integer(slc.getInnerChars());
   }

   /**
    * Setter for the INNER-CHARS writeable attribute.
    *
    * @param ic
    *        new value for the INNER-CHARS
    */
   public void setInnerChars(integer ic)
   {
      setInnerChars(ic.intValue());
   }

   /**
    * Setter for the INNER-CHARS writeable attribute.
    *
    * @param ic
    *        new value for the INNER-CHARS
    */
   public void setInnerChars(int ic)
   {
      slc.setInnerChars(ic);
      frame.pushScreenDefinition();
   }
   
   /**
    * Getter for the INNER-LINES writeable attribute.
    *
    * @return current value of INNER-LINES attribute
    */
   public integer getInnerLines()
   {
      return new integer(slc.getInnerLines());
   }
   
   /**
    * Setter for the INNER-LINES writeable attribute.
    *
    * @param il
    *        new value for the INNER-LINES
    */
   public void setInnerLines(integer il)
   {
      setInnerLines(il.intValue());
   }

   /**
    * Setter for the INNER-LINES writeable attribute.
    *
    * @param il
    *        new value for the INNER-LINES
    */
   public void setInnerLines(int il)
   {
      slc.setInnerLines(il);
      frame.pushScreenDefinition();
   }

   /**
    * Getter for the MULTIPLE read-only attribute.
    * 
    * @return  value of MULTIPLE attribute
    */
   public logical isMultiple()
   {
      return new logical(slc.isMultiple());
   }

   /**
    * Set MULTIPLE attribute.
    * 
    * @param   multiple
    *          New value for the attribute.
    */
   public void setMultiple(boolean multiple)
   {
      slc.setMultiple(multiple);
      frame.pushScreenDefinition();
   }

   /**
    * Set MULTIPLE attribute.
    * 
    * @param   multiple
    *          New value for the attribute.
    */
   public void setMultiple(logical multiple)
   {
      setMultiple(multiple.booleanValue());
   }

   /**
    * Convenience method to turning off MULTIPLE attribute.
    * 
    * @param   single
    *          New value for the attribute.
    */
   public void setSingle(boolean single)
   {
      setMultiple(!single);
   }
   
   /**
    * Setter for the LIST-ITEMS writeable attribute.
    *
    * @param l
    *        new value for the LIST-ITEMS
    */
   public void setListItems(String l)
   {
      setListItems(new character(l));
   }
   
   /**
    * Checks whether the selection list must draw the horizontal scroll-bar if
    * the inner size is more than visible one.
    * 
    * @return   The value of the SCROLLBAR-HORIZONTAL attribute.
    */
   public logical isScrollbarHorizontal()
   {
      return new logical(slc.isScrollbarHorizontal());
   }
   
   /**
    * Resets the horizontal scroll-bar flag with the new one.
    * 
    * @param    scrollbar
    *           New value for the SCROLLBAR-HORIZONTAL attribute.
    */
   public void setScrollbarHorizontal(logical scrollbar)
   {
      setScrollbarHorizontal(scrollbar.booleanValue());
   }
   
   /**
    * Resets the horizontal scroll-bar flag with the new one.
    * 
    * @param    scrollbar
    *           New value for the SCROLLBAR-HORIZONTAL attribute.
    */
   public void setScrollbarHorizontal(boolean scrollbar)
   {
      slc.setScrollbarHorizontal(scrollbar);
   }
   
   /**
    * Checks whether the selection list must draw the vertical scroll-bar if
    * the inner size is more than visible one.
    * 
    * @return   The value of the SCROLLBAR-VERTICAL attribute.
    */
   public logical isScrollbarVertical()
   {
      return new logical(slc.isScrollbarVertical());
   }
   
   /**
    * Resets the vertical scroll-bar flag with the new one.
    * 
    * @param    scrollbar
    *           New value for the SCROLLBAR-VERTICAL attribute.
    */
   public void setScrollbarVertical(logical scrollbar)
   {
      setScrollbarVertical(scrollbar.booleanValue());
   }
   
   /**
    * Resets the vertical scroll-bar flag with the new one.
    * 
    * @param    scrollbar
    *           New value for the SCROLLBAR-VERTICAL attribute.
    */
   public void setScrollbarVertical(boolean scrollbar)
   {
      slc.setScrollbarVertical(scrollbar);
   }
   
   /**
    * Implements the IS-SELECTED() widget method.
    *
    * @param  item
    *         a string representation of an item to check
    * 
    * @return true if the specified item is selected
    */
   public logical isSelected(character item)
   {
      integer index = lookup(item);
      
      if (index.isUnknown() || index.intValue() == 0)
         return new logical(false);
      
      return isSelected(index);
   }
   
   /**
    * Implements the IS-SELECTED() widget method.
    *
    * @param  index
    *         1-based index of an item to check
    * 
    * @return true if the specified item is selected
    */
   public logical isSelected(integer index)
   {
      return new logical(isSelected(index.intValue()));
   }

   /**
    * Helper methods with common selected index lookup code.
    *
    * @param  index
    *         1-based index of an item to check
    * 
    * @return <code>true</code> if the specified item is selected
    */
   private boolean isSelected(int index)
   {
      ControlSetItem[] items = ((ControlSetConfig) config).getItems();
      
      if (index < 1 || index > items.length)
         return false;

      // adjust index to make it 0-based
      index--;
      
      int [] selection = frame.getSelection(this);
      
      if (selection == null)
         return false;

      // This is a short array, so a sort/binary search does not improve 
      // performance because average number of operations is too small to
      // justify overhead. 
      for (int j = 0; j < selection.length; j++)
      {
         if(selection[j] == index)
            return true;
      }
      
      return false;
   }
}
