/*
** Module   : CommonHandleChain.java
** Abstract : Defines common APIs used by handles which allow chaining.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 CA  20130112 Created initial version.
** 002 CA  20130221 Added setters for NAME attribute.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.ui.GenericWidget;

/**
 * Exposes the following methods or attributes, common for the window, widget,
 * frame and procedure handles:
 * <table>
 * <tr>
 *    <th>attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>NAME</td>
 *    <td>{@link #getName()}</td>
 * </tr>
 * <tr>
 *    <td>PREV-SIBLING</td>
 *    <td>{@link #getPrevSibling()}</td>
 * </tr>
 * <tr>
 *    <td>NEXT-SIBLING</td>
 *    <td>{@link #getNextSibling()}</td>
 * </tr>
 * <tr>
 *    <td>PRIVATE-DATA</td>
 *    <td>{@link #getPrivateData()}, {@link #setPrivateData(character)} and
 *        {@link #setPrivateData(String)}
 *    </td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrap(handle)} API, which, depending on the 
 * handle's state and type, it either returns an 
 * {@link ExternalProgramWrapper} instance (for procedure handles) or an 
 * {@link GenericWidget} instance (for widget, window and frame handles), or  
 * a dynamic proxy using {@link handle#invalidAttrAccessProxy}.
 * <p>
 * All implementations which do not support one or more of the NAME, PRIVATE-DATA, NEXT-SIBLING
 * or PREV-SIBLING attributes should call {@link handle#invalidAttribute}. If an attribute is
 * supported only in read-only mode or the attribute is not supported but its setter is called, 
 * then its setter implementation must explicitly call {@link handle#readOnlyError}.
 * <p>
 * This interface must be implemented by the following resources: BUFFER, BUFFER-FIELD, PROCEDURE,
 * QUERY, SAX-ATTRIBUTES, SAX-READER, SAX-WRITER, SERVER, SERVER-SOCKET, SOAP-HEADER, 
 * SOAP-HEADER-ENTRYREF, SOCKET, TEMP-TABLE, widgets, X-DOCUMENT, X-NODEREF. Following is a list
 * of attributes not supported by certain resources:
 * <ul>
 * <li>NEXT-SIBLING: BUFFER-FIELD, SAX-ATTRIBUTES, SAX-READER, SAX-WRITER, TEMP-TABLE, X-DOCUMENT,
 *                   X-NODEREF</li>
 * <li>PREV-SIBLING: BUFFER, BUFFER-FIELD, SAX-ATTRIBUTES, SAX-READER, SAX-WRITER, TEMP-TABLE, 
 *                   X-DOCUMENT, X-NODEREF</li>
 * <li>NAME: SAX-WRITER; writable only for: TEMP-TABLE, widgets</li>
 * 
 * </ul>
 */
public interface CommonHandleChain
extends CommonHandle
{
   /**
    * Get the <code>name</code> attribute of handle.
    * 
    * @return   See above.
    */
   public character getName();

   /**
    * Set the <code>name</code> attribute of handle.
    * 
    * @param name
    *        The name to be set.
    */
   public void setName(String name);
   
   /**
    * Set the <code>name</code> attribute of handle.
    * 
    * @param name
    *        The name to be set.
    */
   public void setName(character name);

   /**
    * Get the <code>next-sibling</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public handle getNextSibling();

   /**
    * Get the <code>prev-sibling</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public handle getPrevSibling();

   /**
    * Get the <code>private-data</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public character getPrivateData();

   /**
    * Set the <code>private-data</code> attribute of this handle.
    * 
    * @param    data
    *           The new private-data value. 
    */
   public void setPrivateData(character data);

   /**
    * Set the <code>private-data</code> attribute of this handle.
    * 
    * @param    data
    *           The new private-data value. 
    */
   public void setPrivateData(String data);

}
