/*
** Module   : FileSystemOps.java
** Abstract : file system utility methods
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -------------------Description--------------------
** 001 GES 20050706   @21665 Created initial version, supporting common
**                           language statements and built-in functions
**                           for manipulating and querying the file
**                           system.
** 002 GES 20050728   @21922 Added searchPath to support the built-in
**                           search() function.
** 003 GES 20050831   @22425 Added FILE-INFO system handle support.
** 004 GES 20060123   @24033 Moved to a context-local implementation.
** 005 GES 20060211   @24540 Split the implementation from the Progress
**                           interface/semantic. The FileSystem interface
**                           was designed to allow this. This enables
**                           a remote implementation of the interface.
** 006 GES 20060322   @25202 Search path must be aware of the mapping
**                           between Progress source filenames and the
**                           Java equivalent.
** 007 GES 20080123   @36900 Changed getValue to toStringMessage which is
**                           safer. Added null checks everywhere to make
**                           the code safer.
** 008 GES 20080321   @37631 Added full support for nearly all remaining
**                           FILE-INFO features.  The remaining features
**                           are stubbed out.
** 009 GES 20110712          Initialize the case-sensitivity setting of the
**                           legacy system to allow searching to honor it.
** 010 GES 20111020          Modified per-session initialization to detect
**                           when the client code is running inside the server
**                           process AND to initialize for local redirection
**                           instead of using a remote object.
** 011 CS  20121104          Implement PROPATH assignment.
** 012 CA  20130112          Added asHandle, isValid, getType and isUnknown  
**                           methods, accessed by the FILE-INFO handle. 
**                           FILE-NAME attribute is converted to getFileName, 
**                           as it needs to be consistent with the procedure's
**                           FILE-NAME attribute.
** 013 CA  20130221          getType getter for TYPE attribute was renamed to getResourceType.
*/

package com.goldencode.p2j.util;

import java.util.*;
import java.io.*;
import com.goldencode.p2j.net.*;
import com.goldencode.p2j.security.ContextLocal;
import com.goldencode.proxy.StaticProxy;

/**
 * Progress compatible static file-system utility methods.
 * <p>
 * This supports some common global variables that are called built-in
 * functions in Progress but which take no parameters, not even empty
 * parenthesis. These variables are read-only (they cannot be assigned).
 * <p>
 * A partial implementation of the <code>FILE-INFO</code> system handle is
 * available but only the <code>FILE-NAME</code> and <code>FILE-SIZE</code>
 * attributes are supported.
 * <p>
 * The following is the list of supported Progress 4GL features:
 * <p>
 * <pre>
 * Progress Variable                    Java Equivalent
 * ---------------------                ----------------------
 * os-error                             {@link #getLastError}
 * os-drives                            {@link #getRootList}
 * os-getenv                            {@link #getProperty}
 * </pre>
 * <p>
 * <pre>
 * Language Statement                   Java Equivalent
 * ---------------------                ----------------------
 * os-append                            {@link #append}
 * os-copy                              {@link #copy}
 * os-create-dir                        {@link #mkdir}
 * os-delete                            {@link #delete}
 * os-rename                            {@link #rename}
 * </pre>
 * <p>
* <pre>
 * Function                             Java Equivalent
 * ---------------------                ----------------------
 * search                               {@link #searchPath}
 * </pre>
 * <p>
* <pre>
 * System Handle:Attribute              Java Equivalent
 * -------------------------            ----------------------
 * file-info:file-create-date           {@link #fileInfoGetCreationDate} (get)
 * file-info:file-create-time           {@link #fileInfoGetCreationTime} (get)
 * file-info:file-mod-date              {@link #fileInfoGetModDate}      (get)
 * file-info:file-mod-time              {@link #fileInfoGetModTime}      (get)
 * file-info:file-name                  {@link #initFileInfo}            (set)
 * file-info:file-name                  {@link #getFileName}             (get)
 * file-info:file-size                  {@link #fileInfoGetSize}         (get)
 * file-info:file-type                  {@link #fileInfoGetType}         (get)
 * file-info:full-pathname              {@link #fileInfoGetFullPath}     (get)
 * file-info:pathname                   {@link #fileInfoGetPathName}     (get)
 * </pre>
 * <p>
 * This class implements the Progress interface/semantic. However the actual
 * implementation of this function is provided by a network object that
 * implements the {@link FileSystem} interface.  This means that the actual
 * operations occur on the remote node associated with this user's session.
 * Please see {@link FileSystemDaemon} and the <code>RemoteObject</code> in
 * the <code>net</code> package.
 */
public class FileSystemOps
{
   /** Stores context-local state variables. */
   private static ContextContainer work = new ContextContainer(); 

   /**
    * Get a the instance for the FILE-INFO system handle.  Is obtained using
    * a call to the {@link StaticProxy#obtain(Class, Class[])}, using the
    * {@link CommonFileInfo} interface and its methods implemented by these
    * classes: {@link FileSystemOps}.
    * 
    * @return   See above.
    */
   public static handle asHandle()
   {
      Class[] classes = { FileSystemOps.class };
      CommonFileInfo proxy = StaticProxy.obtain(CommonFileInfo.class,
                                                classes);
      return new handle(proxy);
   }   
   
   /**
    * Implementation for the {@link WrappedResource#isValid()} API.
    * 
    * @return   Always true.
    */
   public static boolean isValid()
   {
      return true;
   }
   
   /**
    * Implementation for the {@link WrappedResource#isUnknown()} API.
    * 
    * @return   Always false.
    */
   public static boolean isUnknown()
   {
      return false;
   }
   
   /**
    * API needed to implement read-only attribute assignment (a 4GL 
    * "feature").
    * 
    * @param    attribute
    *           The attribute's name.
    *           
    * @see      handle#readOnlyError(handle, String)
    */
   public static void readOnlyError(String attribute)
   {
      handle.readOnlyError(asHandle(), attribute);
   }
   
   /**
    * Get the error code of the last operating system operation.
    *
    * @return   The Progress-compatible error code.
    */
   public static integer getLastError()
   {
      return new integer(work.obtain().lastError);
   }
   
   /**
    * Sets the error code of the last operating system operation.
    *
    * @param    err
    *           The Progress-compatible error code.
    */
   public static void setLastError(int err)
   {
      work.obtain().lastError = err;
   }
   
   /**
    * Search for the specified file and initialize the related data
    * structures if found.  Relative filenames are searched for via the 
    * PROPATH.  If <code>name</code> is <code>unknown</code> or not found,
    * the current data structures are cleared.
    *
    * @param    name
    *           Relative or absolute filename to obtain information on.
    */
   public static void initFileInfo(character name)
   {
      if (name == null || name.isUnknown())
      {
         work.obtain().info = null;
      }
      
      initFileInfo(name.toStringMessage());
   }
   
   /**
    * Search for the specified file and initialize the related data
    * structures if found.  Relative filenames are searched for via the 
    * {@link EnvironmentOps#getSearchPath}.  If <code>name</code> is
    * <code>unknown</code> or not found, the current data structures are
    * cleared.
    *
    * @param    name
    *           Relative or absolute filename to obtain information on.
    */
   public static void initFileInfo(String name)
   {
      WorkArea wa = work.obtain();
      wa.info = (name == null) ? null : wa.fs.accessFileInfo(name);
   }
   
   /**
    * If the current file information data structures are initialized, return
    * the file name, otherwise return the <code>unknown value</code>.
    *
    * @return   The file name of the current file being inspected as input
    *           by the caller (even if the file could not be found) or the
    *           <code>unknown value</code> if the data structures were never
    *           initialized.
    */
   public static character getFileName()
   {
      WorkArea wa = work.obtain();
      
      if (wa.info == null || wa.info.name == null)
      {
         return new character();
      }
      
      return new character(wa.info.name);
   }
   
   /**
    * If the current file information data structures are initialized, return
    * the absolute or relative path name, otherwise return the
    * <code>unknown value</code>. If the specified file name was absolute,
    * then an absolute pathname will be returned.  Otherwise the relative
    * path that was found via {@link #searchPath} will be returned.
    *
    * @return   The path name of the current file being inspected or the
    *           <code>unknown value</code> if the data structures were never
    *           initialized or if initialization failed.
    */
   public static character fileInfoGetPathName()
   {
      WorkArea wa = work.obtain();
      
      if (wa.info == null || wa.info.pathname == null)
      {
         return new character();
      }
      
      return new character(wa.info.pathname);
   }
   
   /**
    * If the current file information data structures are initialized, return
    * the absolute (canonicalized) path name, otherwise return the
    * <code>unknown value</code>.
    *
    * @return   The absolute path name of the current file being inspected or
    *           the <code>unknown value</code> if the data structures were
    *           never initialized or if initialization failed.
    */
   public static character fileInfoGetFullPath()
   {
      WorkArea wa = work.obtain();
      
      if (wa.info == null || wa.info.fullname == null)
      {
         return new character();
      }
      
      return new character(wa.info.fullname);
   }
   
   /**
    * If the current file information data structures are initialized, return
    * the file's size, otherwise return the <code>unknown value</code>.
    *
    * @return   The size in bytes of the current file being inspected or the
    *           <code>unknown value</code> if the data structures were never
    *           initialized or if initialization failed.
    */
   public static integer fileInfoGetSize()
   {
      WorkArea wa = work.obtain();
      
      if (wa.info == null || wa.info.size == -1)
      {
         return new integer();
      }
      
      return new integer(wa.info.size);
   }
   
   /**
    * If the current file information data structures are initialized, return
    * a string with characters that specify attributes about the file,
    * otherwise return the <code>unknown value</code>.
    * <p>
    * One of the following mutually exclusive characters will be present:
    * <p>
    * <pre>
    * Character   Meaning
    * ---------   -----------------------------
    * D           directory
    * F           file
    * X           unknown
    * </pre>
    * <p>
    * <b>There is no support for "S" (UNIX special devices) or "M" (procedure
    * library members) at this time.</b>
    * <p>
    * One or more of the following characters will be present:
    * <p>
    * <pre>
    * Character   Meaning
    * ---------   -----------------------------
    * H           hidden
    * R           readable
    * W           writable
    * </pre>
    * <p>
    * <b>There is no support for "L" (UNIX symbolic links) or "P" (pipes)
    * at this time.</b>
    *
    * @return   The file attributes of the current file being inspected or
    *           the <code>unknown value</code> if the data structures were
    *           never initialized or if initialization failed.
    */
   public static character fileInfoGetType()
   {
      WorkArea wa = work.obtain();
      
      if (wa.info == null || wa.info.type == null)
      {
         return new character();
      }
      
      return new character(wa.info.type);
   }
   
   /**
    * If the current file information data structures are initialized, return
    * the file's last modification date, otherwise return the 
    * <code>unknown value</code>.
    *
    * @return   The last modification date of the current file being inspected
    *           or the <code>unknown value</code> if the data structures were
    *           never initialized or if initialization failed.
    */
   public static date fileInfoGetModDate()
   {
      WorkArea wa = work.obtain();
      
      if (wa.info == null || wa.info.modification == -1)
      {
         return date.instantiateUnknownDate();
      }
      
      return new date(date.neutralMillisToDate(wa.info.modification, null));
   }
   
   /**
    * If the current file information data structures are initialized, return
    * the file's last modification time (in seconds from midnight), otherwise 
    * return the <code>unknown value</code>.
    *
    * @return   The last modification time in seconds from midnight of the
    *           current file being inspected or the <code>unknown value</code>
    *           if the data structures were never initialized or if
    *           initialization failed.
    */
   public static integer fileInfoGetModTime()
   {
      WorkArea wa = work.obtain();
      
      if (wa.info == null || wa.info.modification == -1)
      {
         return new integer();
      }
      
      // turn the modification millis into a specific date (this drops the
      // time value and leaves just the date as a julian day number)
      Date cd     = date.neutralMillisToDate(wa.info.modification, null);
      date create = new date(cd);
      
      // convert the julian day number into a base millis value for that day
      long base = date.julianDayToMillis(create.longValue());
      
      // get the difference and convert to seconds
      return new integer((wa.info.modification - base) / 1000);
   }
   
   /**
    * If the current file information data structures are initialized, return
    * the file's creation date, otherwise return the 
    * <code>unknown value</code>.
    *
    * @return   Always <code>unknown value</code> at this time.
    */
   public static date fileInfoGetCreationDate()
   {
      return date.instantiateUnknownDate();
   }
   
   /**
    * If the current file information data structures are initialized, return
    * the file's creation time (in seconds from midnight), otherwise 
    * return the <code>unknown value</code>.
    *
    * @return   Always returns <code>unknown value</code> at this time.
    */
   public static integer fileInfoGetCreationTime()
   {
      return new integer();
   }
   
   /**
    * Get the comma-separated list of filesystem roots (drive letters on
    * platforms that support the concept, otherwise returns the equivalent
    * of the empty string).
    *
    * @return   The comma-separated list of drive letters.
    */
   public static character getRootList()
   {
      String roots = null;
      
      try
      {
         roots = work.obtain().fs.getRootList();
      }
      
      catch (IOException ioe)
      {
         setLastError(FileSystem.ERR_IO);
         roots = "";
      }
      
      return new character(roots); 
   }
   
   /**
    * Get the system property (user-defined properties can be specified
    * at the Java command line) associated with the passed key.  If the
    * specified system property does not exist, a '?' is returned.
    *
    * @return   The system property found or '?' if the property does not
    *           exist.
    */
   public static character getProperty(character key)
   {
      if (key == null || key.isUnknown())
      {
         return new character("?");
      }
      
      return getProperty(key.toStringMessage());
   }
   
   /**
    * Get the system property (user-defined properties can be specified
    * at the Java command line) associated with the passed key.  If the
    * specified system property does not exist, a '?' is returned.
    *
    * @return   The system property found or '?' if the property does not
    *           exist.
    */
   public static character getProperty(String key)
   {
      String result = "?";
      
      try
      {
         String str = (key == null) ? null
                                    : work.obtain().fs.getProperty(key);
         
         if (str != null)
         {
            result = str;
         }
      }
      
      catch (Exception exc)
      {
         // return the default '?' if a null pointer, security or illegal
         // argument occurs
      }
      
      return new character(result); 
   }
   
   /**
    * Appends the specified source file to the end of the target file. If
    * any error occurs, no exception will be thrown. The {@link #getLastError}
    * can be used to retrieve the error code.
    *
    * @param    src
    *           The name of the source file.  Must NOT be a directory name.
    * @param    tar
    *           The name of the target file.  If this is a directory name or
    *           if this is a non-existent filename, this method is converted
    *           into a copy operation. 
    */
   public static void append(character src, character tar)
   {
      if (src == null || src.isUnknown() || tar == null || tar.isUnknown())
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         append(src.toStringMessage(), tar.toStringMessage());
      }
   }
   
   /**
    * Appends the specified source file to the end of the target file. If
    * any error occurs, no exception will be thrown. The {@link #getLastError}
    * can be used to retrieve the error code.
    *
    * @param    src
    *           The name of the source file.  Must NOT be a directory name.
    * @param    tar
    *           The name of the target file.  If this is a directory name or
    *           if this is a non-existent filename, this method is converted
    *           into a copy operation. 
    */
   public static void append(character src, String tar)
   {
      if (src == null || src.isUnknown() || tar == null)
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         append(src.toStringMessage(), tar);
      }
   }
   
   /**
    * Appends the specified source file to the end of the target file. If
    * any error occurs, no exception will be thrown. The {@link #getLastError}
    * can be used to retrieve the error code.
    *
    * @param    src
    *           The name of the source file.  Must NOT be a directory name.
    * @param    tar
    *           The name of the target file.  If this is a directory name or
    *           if this is a non-existent filename, this method is converted
    *           into a copy operation. 
    */
   public static void append(String src, character tar)
   {
      if (src == null || tar == null || tar.isUnknown())
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         append(src, tar.toStringMessage());
      }
   }
   
   /**
    * Appends the specified source file to the end of the target file. If
    * any error occurs, no exception will be thrown. The {@link #getLastError}
    * can be used to retrieve the error code.
    *
    * @param    src
    *           The name of the source file.  Must NOT be a directory name.
    * @param    tar
    *           The name of the target file.  If this is a directory name or
    *           if this is a non-existent filename, this method is converted
    *           into a copy operation. 
    */
   public static void append(String src, String tar)
   {
      WorkArea wa = work.obtain();
      
      if (src == null || tar == null)
      {
         wa.lastError = FileSystem.ERR_NO_ERROR;
      }
      else
      {
         wa.lastError = wa.fs.copy(src, tar, false);
      }
   }
   
   /**
    * Copies the specified source file to or over the target file. This
    * method always overwrites the target file.  If any error occurs, no
    * exception will be thrown. The {@link #getLastError} can be used to
    * retrieve the error code.
    *
    * @param    src
    *           The name of the source file.  Must NOT be a directory name.
    * @param    tar
    *           The name of the target file.  If this is a directory name,
    *           the source file name will be used as the target file in
    *           the directory given. 
    */
   public static void copy(character src, character tar)
   {
      if (src == null || src.isUnknown() || tar == null || tar.isUnknown())
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         copy(src.toStringMessage(), tar.toStringMessage());
      }
   }
   
   /**
    * Copies the specified source file to or over the target file. This
    * method always overwrites the target file.  If any error occurs, no
    * exception will be thrown. The {@link #getLastError} can be used to
    * retrieve the error code.
    *
    * @param    src
    *           The name of the source file.  Must NOT be a directory name.
    * @param    tar
    *           The name of the target file.  If this is a directory name,
    *           the source file name will be used as the target file in
    *           the directory given. 
    */
   public static void copy(character src, String tar)
   {
      if (src == null || src.isUnknown() || tar == null)
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         copy(src.toStringMessage(), tar);
      }
   }
   
   /**
    * Copies the specified source file to or over the target file. This
    * method always overwrites the target file.  If any error occurs, no
    * exception will be thrown. The {@link #getLastError} can be used to
    * retrieve the error code.
    *
    * @param    src
    *           The name of the source file.  Must NOT be a directory name.
    * @param    tar
    *           The name of the target file.  If this is a directory name,
    *           the source file name will be used as the target file in
    *           the directory given. 
    */
   public static void copy(String src, character tar)
   {
      if (src == null || tar == null || tar.isUnknown())
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         copy(src, tar.toStringMessage());
      }
   }
   
   /**
    * Copies the specified source file to or over the target file. This
    * method always overwrites the target file.  If any error occurs, no
    * exception will be thrown. The {@link #getLastError} can be used to
    * retrieve the error code.
    *
    * @param    src
    *           The name of the source file.  Must NOT be a directory name.
    * @param    tar
    *           The name of the target file.  If this is a directory name,
    *           the source file name will be used as the target file in
    *           the directory given. 
    */
   public static void copy(String src, String tar)
   {
      WorkArea wa = work.obtain();
      
      if (src == null || tar == null)
      {
         wa.lastError = FileSystem.ERR_NO_ERROR;
      }
      else
      {
         wa.lastError = wa.fs.copy(src, tar, true);
      }
   }
   
   /**
    * Creates each of the directories requested, if it doesn't exist and all
    * path segments up to the last segment do exist.  If the directory name
    * already exists in the filesystem (even if it is a file), this method
    * just returns (this is the Progress behavior).  If any error occurs, no
    * exception will be thrown, but no subsequent directories in the list
    * will be created. The {@link #getLastError} can be used to retrieve
    * the error code.
    *
    * @param    dirs
    *           The list of directories to create.    
    */
   public static void mkdir(String[] dirs)
   {
      if (dirs == null)
         return;
      
      WorkArea wa = work.obtain();      
      wa.lastError = wa.fs.mkdir(dirs);
   }
   
   /**
    * Deletes each of the directories or files specified.  If a non-empty
    * directory name is specified, an error will occur unless the
    * <code>recursive</code> flag is <code>true</code>.  If any error occurs,
    * no exception will be thrown, the deletion process just continues. The
    * {@link #getLastError} can be used to retrieve the error code.
    *
    * @param    list
    *           The list of files and directories to delete.    
    * @param    recursive
    *           Recursively decend through all subdirectories, deleting    
    *           the entire subtree.
    */
   public static void delete(String[] list, boolean recursive)
   {
      if (list == null)
         return;
      
      WorkArea wa = work.obtain();      
      wa.lastError = wa.fs.delete(list, recursive);
   }
   
   /**
    * Renames (and moves if the path is different) the specified source file
    * to the target filename (and path).  If any error occurs, no
    * exception will be thrown. The {@link #getLastError} can be used to
    * retrieve the error code.
    *
    * @param    src
    *           The name of the source file or directory.
    * @param    tar
    *           The name of the target file or directory.  If this is a
    *           directory name for a directory that already exists and the
    *           <code>source</code> is a file, then the source file name will
    *           be used as the target file in the directory given. 
    */
   public static void rename(character src, character tar)
   {
      if (src == null || src.isUnknown() || tar == null || tar.isUnknown())
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         rename(src.toStringMessage(), tar.toStringMessage());
      }
   }
   
   /**
    * Renames (and moves if the path is different) the specified source file
    * to the target filename (and path).  If any error occurs, no
    * exception will be thrown. The {@link #getLastError} can be used to
    * retrieve the error code.
    *
    * @param    src
    *           The name of the source file or directory.
    * @param    tar
    *           The name of the target file or directory.  If this is a
    *           directory name for a directory that already exists and the
    *           <code>source</code> is a file, then the source file name will
    *           be used as the target file in the directory given. 
    */
   public static void rename(character src, String tar)
   {
      if (src == null || src.isUnknown() || tar == null)
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         rename(src.toStringMessage(), tar);
      }
   }
   
   /**
    * Renames (and moves if the path is different) the specified source file
    * to the target filename (and path).  If any error occurs, no
    * exception will be thrown. The {@link #getLastError} can be used to
    * retrieve the error code.
    *
    * @param    src
    *           The name of the source file or directory.
    * @param    tar
    *           The name of the target file or directory.  If this is a
    *           directory name for a directory that already exists and the
    *           <code>source</code> is a file, then the source file name will
    *           be used as the target file in the directory given. 
    */
   public static void rename(String src, character tar)
   {
      if (src == null || tar == null || tar.isUnknown())
      {
         setLastError(FileSystem.ERR_NO_ERROR);
      }
      else
      {
         rename(src, tar.toStringMessage());
      }
   }
   
   /**
    * Renames (and moves if the path is different) the specified source file
    * to the target filename (and path).  If any error occurs, no
    * exception will be thrown. The {@link #getLastError} can be used to
    * retrieve the error code.
    *
    * @param    src
    *           The name of the source file or directory.
    * @param    tar
    *           The name of the target file or directory.  If this is a
    *           directory name for a directory that already exists and the
    *           <code>source</code> is a file, then the source file name will
    *           be used as the target file in the directory given. 
    */
   public static void rename(String src, String tar)
   {
      WorkArea wa = work.obtain();
      
      if (src == null || tar == null)
      {
         wa.lastError = FileSystem.ERR_NO_ERROR;
      }
      else
      {
         wa.lastError = wa.fs.rename(src, tar);
      }
   }
   
   /**
    * Search each directory listed in the {@link EnvironmentOps#getSearchPath}
    * for the given file.  If the file system object exists and is a file,
    * then the filename is returned. If the path contains an empty string or
    * a '.', then the current directory is searched, otherwise only the
    * listed directories are searched.  
    * <p>
    * Special processing is provided in case the original Progress code was
    * checking for the availability of an external procedure.  In this case
    * the filename is checked using the {@link SourceNameMapper#convertName}
    * method.  If it exists, the Java class name is returned.  This will
    * "fake" the converted code into thinking that the external procedure
    * this still exists.
    *
    * @param    filename
    *           The absolute or relative filename for which to search.
    *
    * @return   If the file is found in the current directory, the relative
    *           filename is returned. If the file is absolute or found in any
    *           other path, the full absolute filename is returned.  If the
    *           file is not found, the filename is the 
    *           <code>unknown value</code> or the referenced file system
    *           object is not a file, the <code>unknown value</code> is
    *           returned.
    */
   public static character searchPath(character filename)
   {
      if (filename == null || filename.isUnknown())
      {
         return new character();
      }
      
      return searchPath(filename.toStringMessage());
   }
   
   /**
    * Search each directory listed in the {@link EnvironmentOps#getSearchPath}
    * for the given file.  If the file system object exists and is a file,
    * then the filename is returned. If the path contains an empty string or
    * a '.', then the current directory is searched, otherwise only the
    * listed directories are searched.  
    * <p>
    * Special processing is provided in case the original Progress code was
    * checking for the availability of an external procedure.  In this case
    * the filename is checked using the {@link SourceNameMapper#convertName}
    * method.  If it exists, the Java class name is returned.  This will
    * "fake" the converted code into thinking that the external procedure
    * this still exists.
    *
    * @param    filename
    *           The absolute or relative filename for which to search.
    *
    * @return   If the file is found in the current directory, the relative
    *           filename is returned. If the file is absolute or found in any
    *           other path, the full absolute filename is returned.  If the
    *           file is not found or the referenced file system object is not
    *           a file, the <code>unknown value</code> is returned.
    */
   public static character searchPath(String filename)
   {
      if (filename == null)
      {
         return new character();
      }
      
      // if the Progress code was checking for the availability of an
      // external procedure, we have to "fake" it out into thinking that
      // this still exists, try a lookup first
      String cls = SourceNameMapper.convertName(filename);
      
      if (cls != null)
      {
         // this is a program name, return the Java class name instead
         return new character(cls);
      }
      
      String result = work.obtain().fs.searchPath(filename);
      
      // did we find a match?
      if (result == null)
      {
         // no, return unknown value
         return new character();
      }

      return new character(result);
   }
   
   /**
    * Get the type of its associated handle.
    * 
    * @return   Always return the PSEUDO-WIDGET value.
    */
   public static character getResourceType()
   {
      return new character("PSEUDO-WIDGET");
   }

   /**
    * Sets the Progress search path on the TARGET P2J system which is used for
    * the Progress compatible environment. This corresponds to the assignment of
    * the "PROPATH" variable in Progress and represents a mechanism to override
    * the default that is available from the directory.
    * 
    * @param pathList
    *        The array of directory names to set.
    * @param caseSens
    *        Logical value representing the legacy file system being
    *        case-sensitive or not.
    */
   static void setSearchPath(String[] pathList, boolean caseSens)
   {
      if (pathList != null)
      {
         work.obtain().fs.setSearchPath(pathList, 
                                        caseSens);
      }
   }
            
   /** 
    * Stores global data relating to the state of the current context.
    */
   private static class WorkArea
   {  
      /** Stores the Progress error code for the last operation. */ 
      private int lastError = 0; 
      
      /** File information for the current file being examined. */ 
      private FileInfo info = null; 
      
      /** Remote instance of our file system worker. */ 
      private FileSystem fs = null; 
   }
   
   /**
    * Simple container that stores and returns a context-local instance of
    * the global work area.
    */
   private static class ContextContainer
   extends ContextLocal
   {
      /**
       * Obtains the context-local instance of the contained global work
       * area.
       *
       * @return   The work area associated with this context.
       */
      public WorkArea obtain() 
      {
         return (WorkArea) this.get();
      }   
      
      /**
       * Initializes the work area, the first time it is requested within a
       * new context.
       *
       * @return   The newly instantiated work area.
       */
      protected synchronized Object initialValue()
      {
         WorkArea wa = new WorkArea();
         
         // obtain the our proxy, server-side code is left-side by convention
         wa.fs = (FileSystem) RemoteObject.obtainInstance(FileSystem.class,
                                                          true);
         
         // push down our search path to ensure that it is always there
         String[] paths    = EnvironmentOps.getSearchPathList();
         boolean  caseSens = EnvironmentOps.getLegacyCaseSensitive();
         
         if (paths != null || !caseSens)
            wa.fs.setSearchPath(paths, caseSens);
         
         return wa;
      }   
   }   
}
