/*
** Module   : HandleChain.java
** Abstract : An abstract class which provides an  implementation for the most
**            common methods from CommonHandleChain.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CS  20130207 Created initial version.
** 002 CS  20130215 TYPE attribute must be saved as a string, not character.
** 003 CA  20130221 Extracted TYPE, unknown and readOnlyError into a separate superclass.
*/

package com.goldencode.p2j.util;

/**
 * This acts as a superclass which implements the most common methods of the
 * CommonHandleChain interface:
 * <table>
 * <tr>
 *    <th>attribute or base method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>NAME</td>
 *    <td>{@link #getName()}</td>
 * </tr>
 * <tr>
 *    <td>PREV-SIBLING</td>
 *    <td>{@link #getPrevSibling()}</td>
 * </tr>
 * <tr>
 *    <td>NEXT-SIBLING</td>
 *    <td>{@link #getNextSibling()}</td>
 * </tr>
 * <tr>
 *    <td>PRIVATE-DATA</td>
 *    <td>{@link #getPrivateData()}, {@link #setPrivateData(character)} and
 *        {@link #setPrivateData(String)}
 *    </td>
 * </tr>
 * </table>
 * <p>
 * The abstract class also provides setters for the implemented attributes 
 * which are not defined inside the interfaces to be used for initializing of
 * those attributes
 */
public abstract class HandleChain
extends HandleResource
implements CommonHandleChain,
           Deletable
{
   // TODO Extend this to other classes which need the common implemented methods
   
   /** Variable for holding the name attribute*/
   private final character name = new character();
   
   /** Variable for holding the privateData attribute*/
   private final character privateData = new character();
   
   /** Variable for holding the PREV-SIBLING attribute */
   private handle prevSibling = null;
   
   /** Variable for holding the NEXT-SIBLING attribute*/
   private handle nextSibling = null;

   /**
    * Worker to be implemented by each resource.  Called by {@link #delete()}.
    */
   protected abstract void resourceDelete();
   
   /**
    * Show a specific error, as this read-only <code>attribute</code> was used
    * on the right-side of an assignment.
    *  
    * @param    attribute
    *           The read-only attribute.
    */
   public void readOnlyError(String attribute)
   {
      handle.readOnlyError(new handle(this), attribute);
   }

   /**
    * Perform actual delete of an resource. At the time of this call, it is assumed the resource
    * is valid for deletion (the handle and the resource are both valid).
    */
   public void delete()
   {
      // TODO: implement this
   }
   
   /**
    * Get the <code>name</code> attribute of handle.
    * 
    * @return   See above.
    */
   public character getName()
   {
      if (!hasName())
      {
         invalidAttribute("NAME");

         return new character();
      }

      return name;
   }
   
   /**
    * Sets the name of this handle.This method is outside interfaces and only 
    * meant for initializing this within the implementation.
    * 
    * @param name
    *        The handle type to be set.
    */
   public void setName(character name)
   {
      if (hasNameReadOnly())
      {
         readOnlyError("NAME");
         
         return;
      }

      this.name.assign(name); 
   }
   
   /**
    * Sets the name of this handle.This method is outside interfaces and only 
    * meant for initializing this within the implementation.
    * 
    * @param name
    *        The handle type to be set.
    */
   public void setName(String name)
   {
      setName(new character(name)); 
   }

   /**
    * Get the <code>next-sibling</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public handle getNextSibling()
   {
      if (!hasNextSibling())
      {
         invalidAttribute("PREV-SIBLING");
         
         return new handle();
      }

      return nextSibling;
   }
   
   /**
    * Sets the NEXT-SIBLING attribute of this handle.This method is outside 
    * interfaces and only meant for initializing this within the implementation.
    * 
    * @param nextSibling
    *        The handle next sibling to be set.
    */
   public void setNextSibling(handle nextSibling)
   {
      if (!hasNextSibling())
         return;

      this.nextSibling = nextSibling;
   }

   /**
    * Get the <code>prev-sibling</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public handle getPrevSibling()
   {
      if (!hasPrevSibling())
      {
         invalidAttribute("NEXT-SIBLING");
         
         return new handle();
      }

      return prevSibling;
   }
   
   /**
    * Sets the PREV-SIBLING attribute of this handle.This method is outside 
    * interfaces and only meant for initializing this within the implementation.
    * 
    * @param prevSibling
    *        The handle previous sibling to be set.
    */
   public void setPrevSibling(handle prevSibling)
   {
      if (!hasPrevSibling())
         return;
      
      this.prevSibling = prevSibling;
   }

   /**
    * Get the <code>private-data</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public character getPrivateData()
   {
      if (!hasPrivateData())
      {
         invalidAttribute("PRIVATE-DATA");
         
         return new character();
      }
      
      return privateData;
   }

   /**
    * Set the <code>private-data</code> attribute of this handle.
    * 
    * @param    data
    *           The new private-data value. 
    */
   public void setPrivateData(character data)
   {
      if (!hasPrivateData())
      {
         readOnlyError("PRIVATE-DATA");
         
         return;
      }

      this.privateData.assign(data);
   }

   /**
    * Set the <code>private-data</code> attribute of this handle.
    * 
    * @param    data
    *           The new private-data value. 
    */
   public void setPrivateData(String data)
   {
      setPrivateData(new character(data));
   }
   
   /**
    * Check if this resource supports the NEXT-SIBLING attribute.
    * 
    * @return   <code>true</code> if the resource has the attribute.
    */
   protected boolean hasNextSibling()
   {
      return true;
   }
   
   /**
    * Check if this resource supports the PREV-SIBLING attribute.
    * 
    * @return   <code>true</code> if the resource has the attribute.
    */
   protected boolean hasPrevSibling()
   {
      return true;
   }
   
   /**
    * Check if this resource supports the NAME attribute in read-only mode.
    * 
    * @return   <code>true</code> if the resource has the attribute in r/o mode.
    */
   protected boolean hasNameReadOnly()
   {
      return true;
   }
   
   /**
    * Check if this resource supports the NAME attribute.
    * 
    * @return   <code>true</code> if the resource has the attribute.
    */
   protected boolean hasName()
   {
      return true;
   }
   
   /**
    * Check if this resource supports the PRIVATE-DATA attribute.
    * 
    * @return   <code>true</code> if the resource has the attribute.
    */
   protected boolean hasPrivateData()
   {
      return true;
   }
}
