/*
** Module   : SaxAttributes
** Abstract : SAX attributes specific interface.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130208 Created initial version.
** 002 CA  20130221 Added ADM-DATA and UNIQUE-ID support. 
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Interface describing SAX-Attributes specific features of the SAX objects.
 */
public interface SaxAttributes
extends SaxEntity,
        ADMData,
        UniqueID
{
   /**
    * Gets the numbers of the enties in the attributes object.
    * 
    * @return   The integer value of the num-items attribute.
    */
   public integer getNumItems();
   
   /**
    * Gets the value of the atribute with given index.
    * 
    * @param    index
    *           The 1-based index of the attribute to get.
    * 
    * @return   The character value of the attribute with given number.
    */
   public character getValueByIndex(double index);

   /**
    * Gets the value of the atribute with given index.
    * 
    * @param    index
    *           The 1-based index of the attribute to get.
    * 
    * @return   The character value of the attribute with given number.
    */
   public character getValueByIndex(NumberType index);

   /**
    * Copies SAX attributes form source provided to the current SAX-Attributes object.
    * 
    * @param    handle
    *           The handle of the SAX-Attribute object to be used as source.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical copySaxAttributes(handle handle);
   
   /**
    * Gets the 1-based index of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The integer value of the given attribute index.
    */
   public integer getIndexByNamespaceName(Text uri, Text localname);
   
   /**
    * Gets the 1-based index of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The integer value of the given attribute index.
    */
   public integer getIndexByNamespaceName(String uri, Text localname);
   
   /**
    * Gets the 1-based index of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The integer value of the given attribute index.
    */
   public integer getIndexByNamespaceName(Text uri, String localname);
   
   /**
    * Gets the 1-based index of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The integer value of the given attribute index.
    */
   public integer getIndexByNamespaceName(String uri, String localname);

   /**
    * Gets the 1-based index of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the index.
    * 
    * @return   The integer value of the given attribute index.
    */
   public integer getIndexByQname(Text qname);
   
   /**
    * Gets the 1-based index of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the index.
    * 
    * @return   The integer value of the given attribute index.
    */
   public integer getIndexByQname(String qname);

   /**
    * Gets the unqualified local name of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute name to get.
    * 
    * @return   The character value of the attribute local name with given number.
    */
   public character getLocalnameByIndex(double index);

   /**
    * Gets the unqualified local name of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute name to get.
    * 
    * @return   The character value of the attribute local name with given number.
    */
   public character getLocalnameByIndex(NumberType index);

   /**
    * Gets the qualified name of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute name to get.
    * 
    * @return   The character value of the attribute qualified name with given number.
    */
   public character getQnameByIndex(double index);

   /**
    * Gets the qualified local name of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute name to get.
    * 
    * @return   The character value of the attribute qualified name with given number.
    */
   public character getQnameByIndex(NumberType index);

   /**
    * Gets the type of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute type to get.
    * 
    * @return   The character value of the attribute type with given number.
    */
   public character getTypeByIndex(double index);

   /**
    * Gets the type of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute type to get.
    * 
    * @return   The character value of the attribute type with given number.
    */
   public character getTypeByIndex(NumberType index);

   /**
    * Gets the type of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute type.
    */
   public character getTypeByNamespaceName(Text uri, Text localname);
   
   /**
    * Gets the type of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute type.
    */
   public character getTypeByNamespaceName(String uri, Text localname);
   
   /**
    * Gets the type of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute type.
    */
   public character getTypeByNamespaceName(Text uri, String localname);
   
   /**
    * Gets the type of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute type.
    */
   public character getTypeByNamespaceName(String uri, String localname);

   /**
    * Gets the type of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the type.
    * 
    * @return   The character value of the given attribute type.
    */
   public character getTypeByQname(character qname);
   
   /**
    * Gets the type of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the type.
    * 
    * @return   The character value of the given attribute type.
    */
   public character getTypeByQname(String qname);

   /**
    * Gets the namespace URI of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the namespace URI to get.
    * 
    * @return   The character value of the namespace URI with given number.
    */
   public character getUriByIndex(double index);

   /**
    * Gets the namespace URI of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the namespace URI to get.
    * 
    * @return   The character value of the namespace URI with given number.
    */
   public character getUriByIndex(NumberType index);

   /**
    * Gets the value of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute.
    */
   public character getValueByNamespaceName(Text uri, Text localname);
   
   /**
    * Gets the value of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute.
    */
   public character getValueByNamespaceName(String uri, Text localname);
   
   /**
    * Gets the value of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute.
    */
   public character getValueByNamespaceName(Text uri, String localname);
   
   /**
    * Gets the value of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute.
    */
   public character getValueByNamespaceName(String uri, String localname);

   /**
    * Gets the value of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the value.
    * 
    * @return   The character value of the given attribute.
    */
   public character getValueByQname(character qname);
   
   /**
    * Gets the value of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the value.
    * 
    * @return   The character value of the given attribute.
    */
   public character getValueByQname(String qname);
   
   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(character name, character value);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(character name, String value);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(character name, character value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(character name, String value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(character name, character value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(character name, String value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(String name, character value);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(String name, String value);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(String name, character value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(String name, String value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(String name, character value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(String name, String value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(NumberType index, character value);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(NumberType index, String value);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(NumberType index, character value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(NumberType index, String value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(NumberType index, character value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(NumberType index, String value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(double index, character value);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(double index, String value);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(double index, character value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(double index, String value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(double index, character value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical updateAttribute(double index, String value, String uri);
}
